@@ -3,11 +3,10 @@ package Parley;
 use strict;
 use warnings;
 
-our $VERSION = '0.58';
+use Parley::Version;  our $VERSION = $Parley::VERSION;
 
 use Catalyst::Runtime '5.70';
 use Catalyst qw/
-    Dumper
     StackTrace
 
     ConfigLoader
@@ -23,20 +22,100 @@ use Catalyst qw/
     Session::State::Cookie
 
     Authentication
-    Authentication::Store::DBIC
-    Authentication::Credential::Password
+    Authorization::Roles
+    Authorization::ACL
 
     I18N
 /;
 
 use Parley::App::Communication::Email qw( :email );
 
-__PACKAGE__->config( version => $VERSION );
+VERSION_MADNESS: {
+    use version;
+    my $vstring = version->new($VERSION)->normal;
+    __PACKAGE__->config(
+        version => $vstring
+    );
+}
+
 __PACKAGE__->setup;
 
 # only show certain log levels in output
 __PACKAGE__->log (Catalyst::Log->new( @{__PACKAGE__->config->{log_levels}} ));
 
+
+# ---- START: ACL RULES ----
+
+#
+# /site
+#
+##__PACKAGE__->deny_access_unless(
+##    '/site/ip_bans',
+##    [$_]
+##)
+##for qw/ip_ban_posting site_moderator/;
+__PACKAGE__->deny_access_unless(
+    '/site/fmodSaveHandler',
+    [qw/site_moderator/]
+);
+__PACKAGE__->deny_access_unless(
+    '/site/ip_bans',
+    sub {
+        my $c = shift;
+        $c->check_any_user_role(
+            qw/site_moderator ip_ban_posting ip_ban_signup ip_ban_login/
+        )
+    }
+);
+__PACKAGE__->deny_access_unless(
+    '/site/ip_info',
+    sub {
+        my $c = shift;
+        $c->check_any_user_role(
+            qw/site_moderator ip_ban_posting ip_ban_signup ip_ban_login/
+        )
+    }
+);
+__PACKAGE__->deny_access_unless(
+    '/site/roleSaveHandler',
+    [qw/site_moderator/]
+);
+__PACKAGE__->deny_access_unless(
+    '/site/saveBanHandler',
+    sub {
+        my $c = shift;
+        $c->check_any_user_role(
+            qw/site_moderator ip_ban_posting ip_ban_signup ip_ban_login/
+        )
+    }
+);
+__PACKAGE__->deny_access_unless(
+    '/site/services',
+    [qw/site_moderator/]
+);
+__PACKAGE__->deny_access_unless(
+    '/site/user',
+    [qw/site_moderator/]
+);
+__PACKAGE__->deny_access_unless(
+    '/site/users',
+    [qw/site_moderator/]
+);
+__PACKAGE__->deny_access_unless(
+    '/site/users_autocomplete',
+    [qw/site_moderator/]
+);
+
+#__PACKAGE__->deny_access_unless(
+#    '/site/users',
+#    [qw/site_moderator/]
+#);
+
+
+# ---- END:   ACL RULES ----
+
+
+
 # I'm sure there's a (better) way to do this by overriding set()/get() in Class::Accessor
 {
     sub set_get {
