@@ -5,6 +5,9 @@ package Parley::Schema::Thread;
 use strict;
 use warnings;
 
+use Parley::Version;  our $VERSION = $Parley::VERSION;
+use DateTime::Format::Pg;
+
 use base 'DBIx::Class';
 
 __PACKAGE__->load_components("PK::Auto", "Core");
@@ -149,7 +152,7 @@ sub PROBABLY_DEAD_last_post_viewed_in_thread :ResultSet {
             },
             {
                 rows        => 1,
-                order_by    => 'created ASC',
+                order_by    => [\'created ASC'],
             }
         );
 
@@ -174,7 +177,7 @@ sub PROBABLY_DEAD_last_post_viewed_in_thread :ResultSet {
         },
         {
             rows        => 1,
-            order_by    => 'created DESC',
+            order_by    => [\'created DESC'],
         }
     );
 
